class BaseLogger(object):

    def __init__(self, service_name='', run_id='', project_id=''):
        self.service_name = service_name
        self.trace_id = run_id
        self.project_id = project_id


    def do_log(self, message, severity, res, local_run, logger):
        message = str(message)
        if local_run is False:
            logger.log_struct({'message': message, 'trace_id': self.trace_id, 'service_name': self.service_name},
                              resource=res, severity=severity)
        else:
            print('{}: {}'.format(severity, message))

    def create_logger(self):
        from google.cloud import logging
        log_client = logging.Client(project=self.project_id)
        logger = log_client.logger(self.service_name)
        return logger

    def update_trace_id(self, new_trace_id):
        self.trace_id = new_trace_id


    def get_project_id(self):
        return self.project_id

    def get_trace_id(self):
        return self.trace_id

    def get_service_name(self):
        return self.service_name


from mms.base_logger import BaseLogger
from google.cloud.logging.resource import Resource


class KubernetesLogger(BaseLogger):

    def __init__(self, service_name='', trace_id='', project_id='', cluster_name='', container_name='',
                 location='', namespace='', local_run=False):
        super().__init__(service_name=service_name,
                         run_id=trace_id,
                         project_id=project_id)
        self.cluster_name = cluster_name,
        self.container_name = container_name,
        self.location = location,
        self.namespace = namespace
        self.local_run = local_run

        if self.local_run is False:
            self.res = Resource(type='k8s_container', labels={
                "cluster_name": self.cluster_name,
                "container_name": self.container_name,
                "project_id": super().get_project_id(),
                "location": self.location,
                "namespace_name": self.namespace,
                "pod_id": ''
            })

        self.logger = super().create_logger()

    def update_trace_id(self, new_trace_id):
        super().update_trace_id(new_trace_id=new_trace_id)

    def info(self, message):
        super().do_log(message=message,
                       severity='INFO',
                       res=self.res,
                       local_run=self.local_run,
                       logger=self.logger)

    def warning(self, message):
        super().do_log(message=message,
                       severity='WARNING',
                       res=self.res,
                       local_run=self.local_run,
                       logger=self.logger)

    def error(self, message):
        super().do_log(message=message,
                       severity='ERROR',
                       res=self.res,
                       local_run=self.local_run,
                       logger=self.logger)

    def critical(self, message):
        super().do_log(message=message,
                       severity='CRITICAL',
                       res=self.res,
                       local_run=self.local_run,
                       logger=self.logger)

    def debug(self, message):
        super().do_log(message=message,
                       severity='DEBUG',
                       res=self.res,
                       local_run=self.local_run,
                       logger=self.logger)


import redis


class Redis(object):
    '''
    Class to handle easy with redis...
    '''

    def create_redis_client(self, host, port, password):
        '''
        Function to create redis client
        '''
        redis_client = redis.Redis(host=host, port=port, password=password)
        return redis_client

    def __init__(self, host, port, password, exp=60*15):
        self.host = host
        self.port = port
        self.password = password
        self.exp = exp
        self.redis_client = self.create_redis_client(self.host, self.port, self.password)

    def set_redis(self, set_key, set_value):
        '''
        :param set_key: specify the key you want to set
        :param set_value: value you want to set
        :return: None
        '''
        self.redis_client.set(set_key, set_value, ex=self.exp)

    def get_redis(self, get_key):
        return self.redis_client.get(get_key)

    def delete_key(self, key):
        self.redis_client.delete(key)




from mms.kms_handler import KmsService


if __name__ == '__main__':

    '''
    
    plain = KmsService.decrypt('v110-4853-facebook-marketing','europe-west4','data-access','data-access-facebook',
                               b'\n$\x00\xe5Y\xc4\x7f@\xbe\x89\x0e\x1ej\xc6\xe8F4\x92\xb7\x82\xe0/\x98\x07\x06\xa4h\x04w\xd9\xfa~\x9c\x152/\xcc\x1a\x12\x81\x02\x00\xd9\xdf ^\xb7\xb2\x05v\xe8\xfah\xfd\xc4\xc9\xf6\xe0\xa4\xa3U\xe6\x17YRk&\xfa\xa7\xed\x8c\xe0\x18\xf2a|J\xe7\xc8\x11\xb4o\x80\x9e[\xc2\xf0\xf5a%\xa8{Cc\xa2\r\xccwYr\xd0F\xeb?\xd0\x19o\r\xf3D|9y\xac\x1c\xf7`k\xfd\xfb\xab<\x046O\xf7\x9c\xe6\x9eD\xd0\x83\xf2\xb3\xe7\x91b\t7g7\xa2\xca}q\xf2s\xed\xe7\xec\x8al^\xb0\x03\x87n\x89\x8a\x1e~s,y9\xad\xeds\x19`\xab\xcf\xb4\x15\x93\xfe\x89\xcc\x1d)n\xf3?U\x1c\x18\xd8u\xe7\xd4J(H:\xdfg\xb9\x18\x82\xcc\xb2Y\x07F\x0fH"\xb8q\xc3k;p\xc1\xa3]\xdcm\xdd\xd7\xee\xe7a\xe7\x11^\x8a\xbf\xd2\xd9\xa1\x85zK`\xd6l\x1f\x84\xf8\x0c\xe6\xbcN\xe98=\x9a:q\xc4\xe7\xe3\x98{\x0bNY\xef\xe1\xa8\xc3z[E\xd6\x10E\xdaW\\\x86,`Z\xc79I\xfc=?\xa9\x86\x8e\x04\xf5\xfc\xbc\x02\x16KEs\x08=m2}')

    print(plain)
    
    '''

    encr = KmsService.encrypt('v110-4853-facebook-marketing','europe-west4','data-access','data-access-facebook', b'this-is-shit')
    print(encr)




from google.cloud import storage
from google.cloud.storage import Blob
import json
import gcsfs
import pandas as pd


class GCS(object):
    def __init__(self, bucket_name=None, project_id=None, cred_file_path=None):
        self.bucket_name = bucket_name

        if cred_file_path is None:
            self.client = storage.Client()
        else:
            self.client = storage.Client.from_service_account_json(cred_file_path)

        if bucket_name is not None:
            self.bucket = self.client.get_bucket(bucket_name)
        else:
            self.bucket = None

        if cred_file_path is None:
            if project_id is None:
                self.fs = gcsfs.GCSFileSystem()
            else:
                self.fs = gcsfs.GCSFileSystem(project=project_id)
        else:
            if project_id is None:
                self.fs = gcsfs.GCSFileSystem(token=cred_file_path)
            else:
                self.fs = gcsfs.GCSFileSystem(project=project_id, token=cred_file_path)

        if project_id is not None:
            self.client.project = project_id

    def __get_bucket_name(self, bucket_name):
        if bucket_name is None:
            bucket_name = self.bucket_name
        return bucket_name

    def __get_bucket(self, bucket_name):
        if bucket_name is None:
            bucket = self.bucket
        else:
            bucket = self.client.get_bucket(bucket_name)
        return bucket

    def read_json(self, file_name, bucket_name=None):
        bucket = self.__get_bucket(bucket_name)
        blob = bucket.get_blob(file_name)
        content = blob.download_as_string()
        decoded = content.decode("utf-8")
        return json.loads(decoded)

    def read_csv(self, file_name, delimiter=';', header='infer', names=None, bucket_name=None):
        bucket_name = self.__get_bucket_name(bucket_name)
        url = "gs://{}/{}".format(bucket_name, file_name)
        df = pd.read_csv(url, sep=delimiter, header=header, names=names, encoding='utf-8')
        return df

    def read_parquet(self, file_name, bucket_name=None):
        bucket_name = self.__get_bucket_name(bucket_name)
        with self.fs.open(bucket_name + '/' + file_name) as f:
            df = pd.read_parquet(f, engine='pyarrow')
        return df

    def list_all_objects(self, prefix=None, bucket_name=None, return_type='URL'):
        bucket = self.__get_bucket(bucket_name)
        blobs = bucket.list_blobs(prefix=prefix)
        bucket_objects = list(blobs)
        if return_type == 'URL':
            bucket_objects_list = []
            for item in bucket_objects:
                bucket_objects_list.append("gs://{}/{}".format(bucket.name, item.name))
            return bucket_objects_list
        else:
            return bucket_objects

    def df_to_csv(self, df, file_name, delimiter=';', header=True, index=False, bucket_name=None):
        df1 = pd.DataFrame(df)
        bucket_name = self.__get_bucket_name(bucket_name)
        file_name = "gs://{}/{}".format(bucket_name, file_name)
        df1.to_csv(file_name, sep=delimiter, header=header, index=index, encoding='utf-8')

    def df_to_parquet(self, df, file_name, bucket_name=None):
        df1 = pd.DataFrame(df)
        bucket_name = self.__get_bucket_name(bucket_name)
        file_name = "gs://{}/{}".format(bucket_name, file_name)
        df1.to_parquet(file_name, engine='pyarrow')

    def df_to_json_list(self, df, file_name, orient='records', bucket_name=None):
        data = pd.DataFrame(df).to_dict('records')
        bucket = self.__get_bucket(bucket_name)
        blob = Blob(file_name, bucket)
        uploading_data_as_string = json.dumps(data)
        return blob.upload_from_string(uploading_data_as_string, content_type='application/json')

    def dict_to_json(self, df, file_name, orient='records', bucket_name=None):
        bucket = self.__get_bucket(bucket_name)
        blob = Blob(file_name, bucket)
        uploading_data_as_string = json.dumps(df)
        return blob.upload_from_string(uploading_data_as_string, content_type='application/json')

    def upload_file(self, local_file, gcs_file_name, bucket_name=None):
        bucket = self.__get_bucket(bucket_name)
        blob = bucket.blob(gcs_file_name)
        blob.upload_from_filename(local_file)

    def download_file(self, gcs_file_name, local_file_name, bucket_name=None):
        bucket = self.__get_bucket(bucket_name)
        blob = bucket.blob(gcs_file_name)
        blob.download_to_filename(local_file_name)

    ''' 
        def delete_bucket(self, bucket_name): 
            bucket = self.__get_bucket(bucket_name)
            _bucket = bucket.get_bucket(bucket_name)
            _bucket.delete()
            print('Bucket {} deleted'.format(_bucket.name))
    '''
if __name__ == '__main__':

    # With default value: (i. e. if you only work within one bucket)
    gcs_handler = GCS(bucket_name='search-sensors-gdwh-raw-data-k8-v4-445679621038-test-jg')
    # Without defaults:
    #gcs_handler = GCS()

    # Read json from GCS into dictionary:
    #df = gcs_handler.read_json('dev/1000testfiles/10/2.json')
    #print(df)

    # Read parquet file from GCS into pandas dataframe:
    #df = gcs_handler.read_parquet('test/prod_depora_2019_04_12_03_03_7b1f7312-5ccf-11e9-b2d7-0a580ac98e3d-PURCH_PRICE.parquet',  bucket_name='search-sensors-gdwh-raw-data-k8-v4-445679621038-test-jg')
    #print(df)

    # Read csv file from GCS into pandas dataframe:
    #df = gcs_handler.read_csv('test/test.csv')
    #print(df)


    # List all objects of a bucket with and without prefix
    # bucket_objects = gcs_handler.list_all_objects(prefix='test/')
    # print(bucket_objects)

    #df = pd.DataFrame({'col1': [1,2,3], 'col2': ['string1', 'string2', 'string3']})
    # df = {'test': 1}
    #cs_handler.df_to_csv(df=df, file_name='test/test.csv')
    # gcs_handler.df_to_parquet(df=df, file_name='test/test.parquet')
    # gcs_handler.df_to_json_list(df=df, file_name='test/test.json')
    # gcs_handler.dict_to_json(df, file_name="test/dict_test.json")


    # Upload a local file:
    # gcs_handler.upload_file(local_file='../README.md', gcs_file_name='test/README.md')

    # Download file from GCS:
    gcs_handler.download_file('test/test.csv', 'hallo.csv')






from google.cloud import kms_v1

def decrypt_symmetric(project_id, location_id, key_ring_id, crypto_key_id,
                      ciphertext):
    """Decrypts input ciphertext using the provided symmetric CryptoKey."""

    #from google.cloud import kms_v1

    # Creates an API client for the KMS API.
    client = kms_v1.KeyManagementServiceClient()

    # The resource name of the CryptoKey.
    name = client.crypto_key_path_path(project_id, location_id, key_ring_id,
                                       crypto_key_id)
    # Use the KMS API to decrypt the data.
    response = client.decrypt(name, ciphertext)
    return response.plaintext



def encrypt_symmetric(project_id, location_id, key_ring_id, crypto_key_id,
                      plaintext):
    """Encrypts input plaintext data using the provided symmetric CryptoKey."""

    from google.cloud import kms_v1

    # Creates an API client for the KMS API.
    client = kms_v1.KeyManagementServiceClient()

    # The resource name of the CryptoKey.
    name = client.crypto_key_path_path(project_id, location_id, key_ring_id,
                                       crypto_key_id)

    # Use the KMS API to encrypt the data.
    response = client.encrypt(name, plaintext)
    return response.ciphertext


class KmsService:

    def __init__(self):
        self.client = kms_v1.KeyManagementServiceClient()

    def decrypt_response(self, project_id, location_id, key_ring_id, crypto_key_id, ciphertext):

        name = self.client.crypto_key_path_path(project=project_id,
                                                location=location_id,
                                                key_ring=key_ring_id,
                                                crypto_key_path=crypto_key_id)

        return self.client.decrypt(name, ciphertext)

    def encrypt_plaintext(self, project_id, location_id, key_ring_id, crypto_key_id, plaintext):

        name = self.client.crypto_key_path_path(project=project_id,
                                                location=location_id,
                                                key_ring=key_ring_id,
                                                crypto_key_path=crypto_key_id)

        return self.client.encrypt(name, plaintext)

    @classmethod
    def init(cls):
        return KmsService()

    @classmethod
    def decrypt(cls, project_id, location_id, key_ring_id, crypto_key_id, ciphertext):

        kms = KmsService.init()
        response = kms.decrypt_response(project_id=project_id,
                                        location_id=location_id,
                                        key_ring_id=key_ring_id,
                                        crypto_key_id=crypto_key_id,
                                        ciphertext=ciphertext)

        return response.plaintext

    @classmethod
    def encrypt(cls, project_id, location_id, key_ring_id, crypto_key_id, plaintext):

        kms = KmsService.init()
        response = kms.encrypt_plaintext(project_id=project_id,
                               location_id=location_id,
                               key_ring_id=key_ring_id,
                               crypto_key_id=crypto_key_id,
                               plaintext=plaintext)

        return response.ciphertext

